@@ -1,9 +1,9 @@
-use crate::rust::vec::Vec;
-use crate::elements;
 use super::{
-	invoke::{Invoke, Identity},
-	misc::{ValueTypeBuilder, ValueTypesBuilder, OptionalValueTypeBuilder},
+	invoke::{Identity, Invoke},
+	misc::{ValueTypeBuilder, ValueTypesBuilder},
 };
+use crate::elements;
+use alloc::vec::Vec;
 
 /// Signature template description
 pub enum Signature {
@@ -12,7 +12,7 @@ pub enum Signature {
 }
 
 /// Signature builder
-pub struct SignatureBuilder<F=Identity> {
+pub struct SignatureBuilder<F = Identity> {
 	callback: F,
 	signature: elements::FunctionType,
 }
@@ -24,13 +24,18 @@ impl SignatureBuilder {
 	}
 }
 
-impl<F> SignatureBuilder<F> where F: Invoke<elements::FunctionType> {
+impl Default for SignatureBuilder {
+	fn default() -> Self {
+		Self::new()
+	}
+}
+impl<F> SignatureBuilder<F>
+where
+	F: Invoke<elements::FunctionType>,
+{
 	/// New builder with callback function specified
 	pub fn with_callback(callback: F) -> Self {
-		SignatureBuilder {
-			callback: callback,
-			signature: elements::FunctionType::default(),
-		}
+		SignatureBuilder { callback, signature: elements::FunctionType::default() }
 	}
 
 	/// Add argument to signature builder
@@ -40,17 +45,14 @@ impl<F> SignatureBuilder<F> where F: Invoke<elements::FunctionType> {
 	}
 
 	/// Add multiple arguments to signature builder
-	pub fn with_params(mut self, value_types: Vec<elements::ValueType>) -> Self {
+	pub fn with_params<I>(mut self, value_types: I) -> Self
+	where
+		I: IntoIterator<Item = elements::ValueType>,
+	{
 		self.signature.params_mut().extend(value_types);
 		self
 	}
 
-	/// Override signature return type
-	pub fn with_return_type(mut self, return_type: Option<elements::ValueType>) -> Self {
-		*self.signature.return_type_mut() = return_type;
-		self
-	}
-
 	/// Start build new argument
 	pub fn param(self) -> ValueTypeBuilder<Self> {
 		ValueTypeBuilder::with_callback(self)
@@ -61,9 +63,29 @@ impl<F> SignatureBuilder<F> where F: Invoke<elements::FunctionType> {
 		ValueTypesBuilder::with_callback(self)
 	}
 
-	/// Start building return type
-	pub fn return_type(self) -> OptionalValueTypeBuilder<Self> {
-		OptionalValueTypeBuilder::with_callback(self)
+	/// Add result to signature builder
+	pub fn with_result(mut self, value_type: elements::ValueType) -> Self {
+		self.signature.results_mut().push(value_type);
+		self
+	}
+
+	/// Add multiple results to signature builder
+	pub fn with_results<I>(mut self, value_types: I) -> Self
+	where
+		I: IntoIterator<Item = elements::ValueType>,
+	{
+		self.signature.results_mut().extend(value_types);
+		self
+	}
+
+	/// Start building new result
+	pub fn result(self) -> ValueTypeBuilder<Self> {
+		ValueTypeBuilder::with_callback(self)
+	}
+
+	/// Start building multiple results
+	pub fn results(self) -> ValueTypeBuilder<Self> {
+		ValueTypeBuilder::with_callback(self)
 	}
 
 	/// Finish current builder
@@ -77,49 +99,42 @@ impl<F> SignatureBuilder<F> where F: Invoke<elements::FunctionType> {
 	}
 }
 
-impl<F> Invoke<Vec<elements::ValueType>> for SignatureBuilder<F>
-	where F: Invoke<elements::FunctionType>
+impl<F, I> Invoke<I> for SignatureBuilder<F>
+where
+	F: Invoke<elements::FunctionType>,
+	I: IntoIterator<Item = elements::ValueType>,
 {
 	type Result = Self;
 
-	fn invoke(self, args: Vec<elements::ValueType>) -> Self {
+	fn invoke(self, args: I) -> Self {
 		self.with_params(args)
 	}
 }
 
-impl<F> Invoke<Option<elements::ValueType>> for SignatureBuilder<F>
-	where F: Invoke<elements::FunctionType>
-{
-	type Result = Self;
-
-	fn invoke(self, arg: Option<elements::ValueType>) -> Self {
-		self.with_return_type(arg)
-	}
-}
-
 impl<F> Invoke<elements::ValueType> for SignatureBuilder<F>
-	where F: Invoke<elements::FunctionType>
+where
+	F: Invoke<elements::FunctionType>,
 {
 	type Result = Self;
 
 	fn invoke(self, arg: elements::ValueType) -> Self {
-		self.with_param(arg)
+		self.with_result(arg)
 	}
 }
 
 /// Type (signature) reference builder (for function/import/indirect call)
-pub struct TypeRefBuilder<F=Identity> {
+pub struct TypeRefBuilder<F = Identity> {
 	callback: F,
 	type_ref: u32,
 }
 
-impl<F> TypeRefBuilder<F> where F: Invoke<u32> {
+impl<F> TypeRefBuilder<F>
+where
+	F: Invoke<u32>,
+{
 	/// New builder chained with specified callback
 	pub fn with_callback(callback: F) -> Self {
-		TypeRefBuilder {
-			callback: callback,
-			type_ref: 0
-		}
+		TypeRefBuilder { callback, type_ref: 0 }
 	}
 
 	/// Set/override of type reference
@@ -129,11 +144,13 @@ impl<F> TypeRefBuilder<F> where F: Invoke<u32> {
 	}
 
 	/// Finish current builder
-	pub fn build(self) -> F::Result { self.callback.invoke(self.type_ref) }
+	pub fn build(self) -> F::Result {
+		self.callback.invoke(self.type_ref)
+	}
 }
 
 /// Multiple signatures builder
-pub struct SignaturesBuilder<F=Identity> {
+pub struct SignaturesBuilder<F = Identity> {
 	callback: F,
 	section: Vec<Signature>,
 }
@@ -145,13 +162,16 @@ impl SignaturesBuilder {
 	}
 }
 
+impl Default for SignaturesBuilder {
+	fn default() -> Self {
+		Self::new()
+	}
+}
+
 impl<F> SignaturesBuilder<F> {
 	/// New builder chained with specified callback
 	pub fn with_callback(callback: F) -> Self {
-		SignaturesBuilder {
-			callback: callback,
-			section: Vec::new(),
-		}
+		SignaturesBuilder { callback, section: Vec::new() }
 	}
 
 	/// Push new signature into the builder output
@@ -166,7 +186,10 @@ impl<F> SignaturesBuilder<F> {
 	}
 }
 
-impl<F> SignaturesBuilder<F> where F: Invoke<SignatureBindings> {
+impl<F> SignaturesBuilder<F>
+where
+	F: Invoke<SignatureBindings>,
+{
 	/// Start building new signature with dedicated builder
 	pub fn signature(self) -> SignatureBuilder<Self> {
 		SignatureBuilder::with_callback(self)
@@ -189,8 +212,10 @@ impl<F> Invoke<u32> for SignaturesBuilder<F> {
 	}
 }
 
-impl<F> SignaturesBuilder<F> where F: Invoke<elements::FunctionSection> {
-
+impl<F> SignaturesBuilder<F>
+where
+	F: Invoke<elements::FunctionSection>,
+{
 	/// Finalize builder spawning element
 	pub fn build(self) -> F::Result {
 		let mut result = elements::FunctionSection::default();
@@ -208,7 +233,10 @@ impl<F> SignaturesBuilder<F> where F: Invoke<elements::FunctionSection> {
 /// Signature bindings
 pub type SignatureBindings = Vec<Signature>;
 
-impl<F> SignaturesBuilder<F> where F: Invoke<SignatureBindings> {
+impl<F> SignaturesBuilder<F>
+where
+	F: Invoke<SignatureBindings>,
+{
 	/// Bind signature list
 	pub fn bind(self) -> F::Result {
 		self.callback.invoke(self.section)
@@ -216,7 +244,7 @@ impl<F> SignaturesBuilder<F> where F: Invoke<SignatureBindings> {
 }
 
 /// Function body (code) builder
-pub struct FuncBodyBuilder<F=Identity> {
+pub struct FuncBodyBuilder<F = Identity> {
 	callback: F,
 	body: elements::FuncBody,
 }
@@ -225,13 +253,16 @@ impl<F> FuncBodyBuilder<F> {
 	/// New body (code) builder given the chain callback
 	pub fn with_callback(callback: F) -> Self {
 		FuncBodyBuilder {
-			callback: callback,
+			callback,
 			body: elements::FuncBody::new(Vec::new(), elements::Instructions::empty()),
 		}
 	}
 }
 
-impl<F> FuncBodyBuilder<F> where F: Invoke<elements::FuncBody> {
+impl<F> FuncBodyBuilder<F>
+where
+	F: Invoke<elements::FuncBody>,
+{
 	/// Set/override entirely with FuncBody struct
 	pub fn with_func(mut self, func: elements::FuncBody) -> Self {
 		self.body = func;
@@ -239,7 +270,10 @@ impl<F> FuncBodyBuilder<F> where F: Invoke<elements::FuncBody> {
 	}
 
 	/// Extend function local list with new entries
-	pub fn with_locals(mut self, locals: Vec<elements::Local>) -> Self {
+	pub fn with_locals<I>(mut self, locals: I) -> Self
+	where
+		I: IntoIterator<Item = elements::Local>,
+	{
 		self.body.locals_mut().extend(locals);
 		self
 	}
@@ -277,7 +311,7 @@ impl Default for FunctionDefinition {
 }
 
 /// Function definition builder
-pub struct FunctionBuilder<F=Identity> {
+pub struct FunctionBuilder<F = Identity> {
 	callback: F,
 	func: FunctionDefinition,
 }
@@ -289,13 +323,19 @@ impl FunctionBuilder {
 	}
 }
 
-impl<F> FunctionBuilder<F> where F: Invoke<FunctionDefinition> {
+impl Default for FunctionBuilder {
+	fn default() -> Self {
+		Self::new()
+	}
+}
+
+impl<F> FunctionBuilder<F>
+where
+	F: Invoke<FunctionDefinition>,
+{
 	/// New function builder with chained callback
 	pub fn with_callback(callback: F) -> Self {
-		FunctionBuilder {
-			callback: callback,
-			func: Default::default(),
-		}
+		FunctionBuilder { callback, func: Default::default() }
 	}
 
 	/// Set that this function is main entry point
@@ -332,7 +372,10 @@ impl<F> FunctionBuilder<F> where F: Invoke<FunctionDefinition> {
 	}
 }
 
-impl<F> Invoke<elements::FunctionType> for FunctionBuilder<F> where F: Invoke<FunctionDefinition> {
+impl<F> Invoke<elements::FunctionType> for FunctionBuilder<F>
+where
+	F: Invoke<FunctionDefinition>,
+{
 	type Result = Self;
 
 	fn invoke(self, signature: elements::FunctionType) -> Self {
@@ -340,7 +383,10 @@ impl<F> Invoke<elements::FunctionType> for FunctionBuilder<F> where F: Invoke<Fu
 	}
 }
 
-impl<F> Invoke<u32> for FunctionBuilder<F> where F: Invoke<FunctionDefinition> {
+impl<F> Invoke<u32> for FunctionBuilder<F>
+where
+	F: Invoke<FunctionDefinition>,
+{
 	type Result = Self;
 
 	fn invoke(self, type_ref: u32) -> Self {
@@ -348,7 +394,10 @@ impl<F> Invoke<u32> for FunctionBuilder<F> where F: Invoke<FunctionDefinition> {
 	}
 }
 
-impl<F> Invoke<elements::FuncBody> for FunctionBuilder<F> where F: Invoke<FunctionDefinition> {
+impl<F> Invoke<elements::FuncBody> for FunctionBuilder<F>
+where
+	F: Invoke<FunctionDefinition>,
+{
 	type Result = Self;
 
 	fn invoke(self, body: elements::FuncBody) -> Self::Result {
@@ -374,23 +423,24 @@ pub fn function() -> FunctionBuilder {
 #[cfg(test)]
 mod tests {
 
-	use super::{signatures, function};
+	use super::{function, signatures};
 	use crate::elements;
 
 	#[test]
 	fn example() {
-		let result = signatures()
-			.type_ref().val(1).build()
-			.build();
+		let result = signatures().type_ref().val(1).build().build();
 
 		assert_eq!(result.entries().len(), 1);
 
 		let result = signatures()
 			.signature()
-				.param().i32()
-				.param().i32()
-				.return_type().i64()
-				.build()
+			.param()
+			.i32()
+			.param()
+			.i32()
+			.result()
+			.i64()
+			.build()
 			.bind();
 
 		assert_eq!(result.len(), 1);
@@ -400,12 +450,34 @@ mod tests {
 	fn func_example() {
 		let func = function()
 			.signature()
-				.param().i32()
-				.return_type().i32()
-				.build()
+			.param()
+			.i32()
+			.result()
+			.i32()
+			.build()
 			.body()
-				.with_instructions(elements::Instructions::empty())
-				.build()
+			.with_instructions(elements::Instructions::empty())
+			.build()
+			.build();
+
+		assert_eq!(func.code.locals().len(), 0);
+		assert_eq!(func.code.code().elements().len(), 1);
+	}
+
+	#[test]
+	fn func_example_multi_result() {
+		let func = function()
+			.signature()
+			.param()
+			.i32()
+			.result()
+			.i32()
+			.result()
+			.i32()
+			.build()
+			.body()
+			.with_instructions(elements::Instructions::empty())
+			.build()
 			.build();
 
 		assert_eq!(func.code.locals().len(), 0);
