@@ -1,9 +1,9 @@
-use crate::rust::vec::Vec;
 use super::invoke::{Identity, Invoke};
 use crate::elements;
+use alloc::vec::Vec;
 
 /// Data segment builder
-pub struct DataSegmentBuilder<F=Identity> {
+pub struct DataSegmentBuilder<F = Identity> {
 	callback: F,
 	// todo: add mapper once multiple memory refs possible
 	mem_index: u32,
@@ -18,11 +18,17 @@ impl DataSegmentBuilder {
 	}
 }
 
+impl Default for DataSegmentBuilder {
+	fn default() -> Self {
+		Self::new()
+	}
+}
+
 impl<F> DataSegmentBuilder<F> {
 	/// New data segment builder inside the chain context
 	pub fn with_callback(callback: F) -> Self {
 		DataSegmentBuilder {
-			callback: callback,
+			callback,
 			mem_index: 0,
 			offset: elements::InitExpr::empty(),
 			value: Vec::new(),
@@ -42,16 +48,16 @@ impl<F> DataSegmentBuilder<F> {
 	}
 }
 
-impl<F> DataSegmentBuilder<F> where F: Invoke<elements::DataSegment> {
+impl<F> DataSegmentBuilder<F>
+where
+	F: Invoke<elements::DataSegment>,
+{
 	/// Finish current builder, spawning resulting struct
 	pub fn build(self) -> F::Result {
-		self.callback.invoke(
-			elements::DataSegment::new(
-				self.mem_index,
-				Some(self.offset),
-				self.value,
-				false,
-			)
-		)
+		self.callback.invoke(elements::DataSegment::new(
+			self.mem_index,
+			Some(self.offset),
+			self.value,
+		))
 	}
 }
