@@ -1,12 +1,12 @@
 use super::{
-	invoke::{Invoke, Identity},
+	invoke::{Identity, Invoke},
 	misc::ValueTypeBuilder,
 };
 
 use crate::elements;
 
 /// Global builder
-pub struct GlobalBuilder<F=Identity> {
+pub struct GlobalBuilder<F = Identity> {
 	callback: F,
 	value_type: elements::ValueType,
 	is_mutable: bool,
@@ -20,11 +20,17 @@ impl GlobalBuilder {
 	}
 }
 
+impl Default for GlobalBuilder {
+	fn default() -> Self {
+		Self::new()
+	}
+}
+
 impl<F> GlobalBuilder<F> {
 	/// New global builder with callback (in chained context)
 	pub fn with_callback(callback: F) -> Self {
 		GlobalBuilder {
-			callback: callback,
+			callback,
 			value_type: elements::ValueType::I32,
 			init_expr: elements::InitExpr::empty(),
 			is_mutable: false,
@@ -55,15 +61,16 @@ impl<F> GlobalBuilder<F> {
 	}
 }
 
-impl<F> GlobalBuilder<F> where F: Invoke<elements::GlobalEntry> {
+impl<F> GlobalBuilder<F>
+where
+	F: Invoke<elements::GlobalEntry>,
+{
 	/// Finalize current builder spawning resulting struct
 	pub fn build(self) -> F::Result {
-		self.callback.invoke(
-			elements::GlobalEntry::new(
-				elements::GlobalType::new(self.value_type, self.is_mutable),
-				self.init_expr,
-			)
-		)
+		self.callback.invoke(elements::GlobalEntry::new(
+			elements::GlobalType::new(self.value_type, self.is_mutable),
+			self.init_expr,
+		))
 	}
 }
 
@@ -88,6 +95,6 @@ mod tests {
 	fn example() {
 		let entry = global().value_type().i32().build();
 		assert_eq!(entry.global_type().content_type(), elements::ValueType::I32);
-		assert_eq!(entry.global_type().is_mutable(), false);
+		assert!(!entry.global_type().is_mutable());
 	}
 }
