@@ -1,5 +1,5 @@
-use io;
-use super::{Deserialize, Serialize, Error, GlobalType, InitExpr};
+use super::{Deserialize, Error, GlobalType, InitExpr, Serialize};
+use crate::io;
 
 /// Global entry in the module.
 #[derive(Clone, Debug, PartialEq)]
@@ -11,19 +11,24 @@ pub struct GlobalEntry {
 impl GlobalEntry {
 	/// New global entry.
 	pub fn new(global_type: GlobalType, init_expr: InitExpr) -> Self {
-		GlobalEntry {
-			global_type: global_type,
-			init_expr: init_expr,
-		}
+		GlobalEntry { global_type, init_expr }
 	}
 	/// Global type.
-	pub fn global_type(&self) -> &GlobalType { &self.global_type }
+	pub fn global_type(&self) -> &GlobalType {
+		&self.global_type
+	}
 	/// Initialization expression (instructions) for global.
-	pub fn init_expr(&self) -> &InitExpr { &self.init_expr }
+	pub fn init_expr(&self) -> &InitExpr {
+		&self.init_expr
+	}
 	/// Global type (mutable).
-	pub fn global_type_mut(&mut self) -> &mut GlobalType { &mut self.global_type }
+	pub fn global_type_mut(&mut self) -> &mut GlobalType {
+		&mut self.global_type
+	}
 	/// Initialization expression (instructions) for global (mutable).
-	pub fn init_expr_mut(&mut self) -> &mut InitExpr { &mut self.init_expr }
+	pub fn init_expr_mut(&mut self) -> &mut InitExpr {
+		&mut self.init_expr
+	}
 }
 
 impl Deserialize for GlobalEntry {
@@ -33,10 +38,7 @@ impl Deserialize for GlobalEntry {
 		let global_type = GlobalType::deserialize(reader)?;
 		let init_expr = InitExpr::deserialize(reader)?;
 
-		Ok(GlobalEntry {
-			global_type: global_type,
-			init_expr: init_expr,
-		})
+		Ok(GlobalEntry { global_type, init_expr })
 	}
 }
 
