@@ -1,14 +1,16 @@
-use alloc::{borrow::ToOwned, vec::Vec, string::String};
 use crate::io;
+use alloc::{borrow::ToOwned, string::String, vec::Vec};
 
-use super::{deserialize_buffer, serialize, Deserialize, Serialize, Error, Uint32, External};
-use super::section::{
-	Section, CodeSection, TypeSection, ImportSection, ExportSection, FunctionSection,
-	GlobalSection, TableSection, ElementSection, DataSection, MemorySection,
-	CustomSection,
+use super::{
+	deserialize_buffer,
+	name_section::NameSection,
+	reloc_section::RelocSection,
+	section::{
+		CodeSection, CustomSection, DataSection, ElementSection, ExportSection, FunctionSection,
+		GlobalSection, ImportSection, MemorySection, Section, TableSection, TypeSection,
+	},
+	serialize, Deserialize, Error, External, Serialize, Uint32,
 };
-use super::name_section::NameSection;
-use super::reloc_section::RelocSection;
 
 use core::cmp;
 
@@ -48,19 +50,17 @@ impl Default for Module {
 impl Module {
 	/// New module with sections
 	pub fn new(sections: Vec<Section>) -> Self {
-		Module {
-			sections: sections, ..Default::default()
-		}
+		Module { sections, ..Default::default() }
 	}
 
 	/// Construct a module from a slice.
 	pub fn from_bytes<T: AsRef<[u8]>>(input: T) -> Result<Self, Error> {
-		Ok(deserialize_buffer::<Module>(input.as_ref())?)
+		deserialize_buffer::<Module>(input.as_ref())
 	}
 
 	/// Serialize a module to a vector.
-	pub fn to_bytes(self) -> Result<Vec<u8>, Error> {
-		Ok(serialize::<Module>(self)?)
+	pub fn into_bytes(self) -> Result<Vec<u8>, Error> {
+		serialize::<Module>(self)
 	}
 
 	/// Destructure the module, yielding sections
@@ -69,7 +69,9 @@ impl Module {
 	}
 
 	/// Version of module.
-	pub fn version(&self) -> u32 { self.version }
+	pub fn version(&self) -> u32 {
+		self.version
+	}
 
 	/// Sections list.
 	///
@@ -93,12 +95,12 @@ impl Module {
 		// Custom sections can be inserted anywhere. Lets always insert them last here.
 		if section.order() == 0 {
 			sections.push(section);
-			return Ok(());
+			return Ok(())
 		}
 
 		// Check if the section already exists.
-		if sections.iter().position(|s| s.order() == section.order()).is_some() {
-			return Err(Error::DuplicatedSections(section.order()));
+		if sections.iter().any(|s| s.order() == section.order()) {
+			return Err(Error::DuplicatedSections(section.order()))
 		}
 
 		// Assume that the module is already well-ordered.
@@ -114,7 +116,9 @@ impl Module {
 	/// Code section reference, if any.
 	pub fn code_section(&self) -> Option<&CodeSection> {
 		for section in self.sections() {
-			if let &Section::Code(ref code_section) = section { return Some(code_section); }
+			if let Section::Code(ref code_section) = *section {
+				return Some(code_section)
+			}
 		}
 		None
 	}
@@ -122,7 +126,9 @@ impl Module {
 	/// Code section mutable reference, if any.
 	pub fn code_section_mut(&mut self) -> Option<&mut CodeSection> {
 		for section in self.sections_mut() {
-			if let Section::Code(ref mut code_section) = *section { return Some(code_section); }
+			if let Section::Code(ref mut code_section) = *section {
+				return Some(code_section)
+			}
 		}
 		None
 	}
@@ -130,7 +136,9 @@ impl Module {
 	/// Types section reference, if any.
 	pub fn type_section(&self) -> Option<&TypeSection> {
 		for section in self.sections() {
-			if let &Section::Type(ref type_section) = section { return Some(type_section); }
+			if let Section::Type(ref type_section) = *section {
+				return Some(type_section)
+			}
 		}
 		None
 	}
@@ -138,7 +146,9 @@ impl Module {
 	/// Types section mutable reference, if any.
 	pub fn type_section_mut(&mut self) -> Option<&mut TypeSection> {
 		for section in self.sections_mut() {
-			if let Section::Type(ref mut type_section) = *section { return Some(type_section); }
+			if let Section::Type(ref mut type_section) = *section {
+				return Some(type_section)
+			}
 		}
 		None
 	}
@@ -146,7 +156,9 @@ impl Module {
 	/// Imports section reference, if any.
 	pub fn import_section(&self) -> Option<&ImportSection> {
 		for section in self.sections() {
-			if let &Section::Import(ref import_section) = section { return Some(import_section); }
+			if let Section::Import(ref import_section) = *section {
+				return Some(import_section)
+			}
 		}
 		None
 	}
@@ -154,7 +166,9 @@ impl Module {
 	/// Imports section mutable reference, if any.
 	pub fn import_section_mut(&mut self) -> Option<&mut ImportSection> {
 		for section in self.sections_mut() {
-			if let Section::Import(ref mut import_section) = *section { return Some(import_section); }
+			if let Section::Import(ref mut import_section) = *section {
+				return Some(import_section)
+			}
 		}
 		None
 	}
@@ -162,24 +176,29 @@ impl Module {
 	/// Globals section reference, if any.
 	pub fn global_section(&self) -> Option<&GlobalSection> {
 		for section in self.sections() {
-			if let &Section::Global(ref section) = section { return Some(section); }
+			if let Section::Global(ref section) = *section {
+				return Some(section)
+			}
 		}
 		None
 	}
 
-		/// Globals section mutable reference, if any.
+	/// Globals section mutable reference, if any.
 	pub fn global_section_mut(&mut self) -> Option<&mut GlobalSection> {
 		for section in self.sections_mut() {
-			if let Section::Global(ref mut section) = *section { return Some(section); }
+			if let Section::Global(ref mut section) = *section {
+				return Some(section)
+			}
 		}
 		None
 	}
 
-
 	/// Exports section reference, if any.
 	pub fn export_section(&self) -> Option<&ExportSection> {
 		for section in self.sections() {
-			if let &Section::Export(ref export_section) = section { return Some(export_section); }
+			if let Section::Export(ref export_section) = *section {
+				return Some(export_section)
+			}
 		}
 		None
 	}
@@ -187,7 +206,9 @@ impl Module {
 	/// Exports section mutable reference, if any.
 	pub fn export_section_mut(&mut self) -> Option<&mut ExportSection> {
 		for section in self.sections_mut() {
-			if let Section::Export(ref mut export_section) = *section { return Some(export_section); }
+			if let Section::Export(ref mut export_section) = *section {
+				return Some(export_section)
+			}
 		}
 		None
 	}
@@ -195,7 +216,9 @@ impl Module {
 	/// Table section reference, if any.
 	pub fn table_section(&self) -> Option<&TableSection> {
 		for section in self.sections() {
-			if let &Section::Table(ref section) = section { return Some(section); }
+			if let Section::Table(ref section) = *section {
+				return Some(section)
+			}
 		}
 		None
 	}
@@ -203,7 +226,9 @@ impl Module {
 	/// Table section mutable reference, if any.
 	pub fn table_section_mut(&mut self) -> Option<&mut TableSection> {
 		for section in self.sections_mut() {
-			if let Section::Table(ref mut section) = *section { return Some(section); }
+			if let Section::Table(ref mut section) = *section {
+				return Some(section)
+			}
 		}
 		None
 	}
@@ -211,7 +236,9 @@ impl Module {
 	/// Data section reference, if any.
 	pub fn data_section(&self) -> Option<&DataSection> {
 		for section in self.sections() {
-			if let &Section::Data(ref section) = section { return Some(section); }
+			if let Section::Data(ref section) = *section {
+				return Some(section)
+			}
 		}
 		None
 	}
@@ -219,7 +246,9 @@ impl Module {
 	/// Data section mutable reference, if any.
 	pub fn data_section_mut(&mut self) -> Option<&mut DataSection> {
 		for section in self.sections_mut() {
-			if let Section::Data(ref mut section) = *section { return Some(section); }
+			if let Section::Data(ref mut section) = *section {
+				return Some(section)
+			}
 		}
 		None
 	}
@@ -227,7 +256,9 @@ impl Module {
 	/// Element section reference, if any.
 	pub fn elements_section(&self) -> Option<&ElementSection> {
 		for section in self.sections() {
-			if let &Section::Element(ref section) = section { return Some(section); }
+			if let Section::Element(ref section) = *section {
+				return Some(section)
+			}
 		}
 		None
 	}
@@ -235,7 +266,9 @@ impl Module {
 	/// Element section mutable reference, if any.
 	pub fn elements_section_mut(&mut self) -> Option<&mut ElementSection> {
 		for section in self.sections_mut() {
-			if let Section::Element(ref mut section) = *section { return Some(section); }
+			if let Section::Element(ref mut section) = *section {
+				return Some(section)
+			}
 		}
 		None
 	}
@@ -243,7 +276,9 @@ impl Module {
 	/// Memory section reference, if any.
 	pub fn memory_section(&self) -> Option<&MemorySection> {
 		for section in self.sections() {
-			if let &Section::Memory(ref section) = section { return Some(section); }
+			if let Section::Memory(ref section) = *section {
+				return Some(section)
+			}
 		}
 		None
 	}
@@ -251,7 +286,9 @@ impl Module {
 	/// Memory section mutable reference, if any.
 	pub fn memory_section_mut(&mut self) -> Option<&mut MemorySection> {
 		for section in self.sections_mut() {
-			if let Section::Memory(ref mut section) = *section { return Some(section); }
+			if let Section::Memory(ref mut section) = *section {
+				return Some(section)
+			}
 		}
 		None
 	}
@@ -259,7 +296,9 @@ impl Module {
 	/// Functions signatures section reference, if any.
 	pub fn function_section(&self) -> Option<&FunctionSection> {
 		for section in self.sections() {
-			if let &Section::Function(ref sect) = section { return Some(sect); }
+			if let Section::Function(ref sect) = *section {
+				return Some(sect)
+			}
 		}
 		None
 	}
@@ -267,7 +306,9 @@ impl Module {
 	/// Functions signatures section mutable reference, if any.
 	pub fn function_section_mut(&mut self) -> Option<&mut FunctionSection> {
 		for section in self.sections_mut() {
-			if let Section::Function(ref mut sect) = *section { return Some(sect); }
+			if let Section::Function(ref mut sect) = *section {
+				return Some(sect)
+			}
 		}
 		None
 	}
@@ -275,7 +316,9 @@ impl Module {
 	/// Start section, if any.
 	pub fn start_section(&self) -> Option<u32> {
 		for section in self.sections() {
-			if let &Section::Start(sect) = section { return Some(sect); }
+			if let Section::Start(sect) = *section {
+				return Some(sect)
+			}
 		}
 		None
 	}
@@ -283,13 +326,14 @@ impl Module {
 	/// Changes the module's start section.
 	pub fn set_start_section(&mut self, new_start: u32) {
 		for section in self.sections_mut().iter_mut() {
-			if let &mut Section::Start(_sect) = section {
+			if let Section::Start(_sect) = *section {
 				*section = Section::Start(new_start);
 				return
 			}
 		}
 		// This should not fail, because we update the existing section above.
-		self.insert_section(Section::Start(new_start)).expect("insert_section should not fail");
+		self.insert_section(Section::Start(new_start))
+			.expect("insert_section should not fail");
 	}
 
 	/// Removes the module's start section.
@@ -299,7 +343,7 @@ impl Module {
 		for (index, section) in sections.iter_mut().enumerate() {
 			if let Section::Start(_sect) = section {
 				rmidx = index;
-				break;
+				break
 			}
 		}
 		if rmidx < sections.len() {
@@ -308,14 +352,10 @@ impl Module {
 	}
 
 	/// Returns an iterator over the module's custom sections
-	pub fn custom_sections(&self) -> impl Iterator<Item=&CustomSection> {
-		self.sections().iter().filter_map(|s| {
-			if let Section::Custom(s) = s {
-				Some(s)
-			} else {
-				None
-			}
-		})
+	pub fn custom_sections(&self) -> impl Iterator<Item = &CustomSection> {
+		self.sections()
+			.iter()
+			.filter_map(|s| if let Section::Custom(s) = s { Some(s) } else { None })
 	}
 
 	/// Sets the payload associated with the given custom section, or adds a new custom section,
@@ -323,7 +363,7 @@ impl Module {
 	pub fn set_custom_section(&mut self, name: impl Into<String>, payload: Vec<u8>) {
 		let name: String = name.into();
 		for section in self.sections_mut() {
-			if let &mut Section::Custom(ref mut sect) = section {
+			if let Section::Custom(ref mut sect) = *section {
 				if sect.name() == name {
 					*sect = CustomSection::new(name, payload);
 					return
@@ -381,7 +421,9 @@ impl Module {
 	/// Call `parse_names` to parse name section
 	pub fn names_section(&self) -> Option<&NameSection> {
 		for section in self.sections() {
-			if let Section::Name(ref sect) = *section { return Some(sect); }
+			if let Section::Name(ref sect) = *section {
+				return Some(sect)
+			}
 		}
 		None
 	}
@@ -392,7 +434,9 @@ impl Module {
 	/// Call `parse_names` to parse name section
 	pub fn names_section_mut(&mut self) -> Option<&mut NameSection> {
 		for section in self.sections_mut() {
-			if let Section::Name(ref mut sect) = *section { return Some(sect); }
+			if let Section::Name(ref mut sect) = *section {
+				return Some(sect)
+			}
 		}
 		None
 	}
@@ -407,26 +451,33 @@ impl Module {
 
 		for i in 0..self.sections.len() {
 			if let Some(name_section) = {
-				let section = self.sections.get(i).expect("cannot fail because i in range 0..len; qed");
+				let section =
+					self.sections.get(i).expect("cannot fail because i in range 0..len; qed");
 				if let Section::Custom(ref custom) = *section {
 					if custom.name() == "name" {
 						let mut rdr = io::Cursor::new(custom.payload());
 						let name_section = match NameSection::deserialize(&self, &mut rdr) {
 							Ok(ns) => ns,
-							Err(e) => { parse_errors.push((i, e)); continue; }
+							Err(e) => {
+								parse_errors.push((i, e));
+								continue
+							},
 						};
 						Some(name_section)
 					} else {
 						None
 					}
-				} else { None }
+				} else {
+					None
+				}
 			} {
 				// todo: according to the spec a Wasm binary can contain only one name section
-				*self.sections.get_mut(i).expect("cannot fail because i in range 0..len; qed") = Section::Name(name_section);
+				*self.sections.get_mut(i).expect("cannot fail because i in range 0..len; qed") =
+					Section::Name(name_section);
 			}
 		}
 
-		if parse_errors.len() > 0 {
+		if !parse_errors.is_empty() {
 			Err((parse_errors, self))
 		} else {
 			Ok(self)
@@ -446,21 +497,23 @@ impl Module {
 				if let Section::Custom(ref custom) = *section {
 					if custom.name().starts_with("reloc.") {
 						let mut rdr = io::Cursor::new(custom.payload());
-						let reloc_section = match RelocSection::deserialize(custom.name().to_owned(), &mut rdr) {
-							Ok(reloc_section) => reloc_section,
-							Err(e) => { parse_errors.push((i, e)); continue; }
-						};
+						let reloc_section =
+							match RelocSection::deserialize(custom.name().to_owned(), &mut rdr) {
+								Ok(reloc_section) => reloc_section,
+								Err(e) => {
+									parse_errors.push((i, e));
+									continue
+								},
+							};
 						if rdr.position() != custom.payload().len() {
 							parse_errors.push((i, io::Error::InvalidData.into()));
-							continue;
+							continue
 						}
 						Some(Section::Reloc(reloc_section))
-					}
-					else {
+					} else {
 						None
 					}
-				}
-				else {
+				} else {
 					None
 				}
 			} {
@@ -468,7 +521,7 @@ impl Module {
 			}
 		}
 
-		if parse_errors.len() > 0 {
+		if !parse_errors.is_empty() {
 			Err((parse_errors, self))
 		} else {
 			Ok(self)
@@ -478,14 +531,20 @@ impl Module {
 	/// Count imports by provided type.
 	pub fn import_count(&self, count_type: ImportCountType) -> usize {
 		self.import_section()
-			.map(|is|
-				is.entries().iter().filter(|import| match (count_type, *import.external()) {
-					(ImportCountType::Function, External::Function(_)) => true,
-					(ImportCountType::Global, External::Global(_)) => true,
-					(ImportCountType::Table, External::Table(_)) => true,
-					(ImportCountType::Memory, External::Memory(_)) => true,
-					_ => false
-				}).count())
+			.map(|is| {
+				is.entries()
+					.iter()
+					.filter(|import| {
+						matches!(
+							(count_type, *import.external()),
+							(ImportCountType::Function, External::Function(_)) |
+								(ImportCountType::Global, External::Global(_)) |
+								(ImportCountType::Table, External::Table(_)) |
+								(ImportCountType::Memory, External::Memory(_))
+						)
+					})
+					.count()
+			})
 			.unwrap_or(0)
 	}
 
@@ -523,45 +582,43 @@ impl Deserialize for Module {
 		let mut magic = [0u8; 4];
 		reader.read(&mut magic)?;
 		if magic != WASM_MAGIC_NUMBER {
-			return Err(Error::InvalidMagic);
+			return Err(Error::InvalidMagic)
 		}
 
 		let version: u32 = Uint32::deserialize(reader)?.into();
 
 		if version != 1 {
-			return Err(Error::UnsupportedVersion(version));
+			return Err(Error::UnsupportedVersion(version))
 		}
 
 		let mut last_section_order = 0;
 
 		loop {
 			match Section::deserialize(reader) {
-				Err(Error::UnexpectedEof) => { break; },
-				Err(e) => { return Err(e) },
+				Err(Error::UnexpectedEof) => break,
+				Err(e) => return Err(e),
 				Ok(section) => {
 					if section.order() != 0 {
-						if last_section_order > section.order() {
-							return Err(Error::SectionsOutOfOrder);
-						} else if last_section_order == section.order() {
-							return Err(Error::DuplicatedSections(last_section_order));
-						}
+						match last_section_order {
+							x if x > section.order() => return Err(Error::SectionsOutOfOrder),
+							x if x == section.order() =>
+								return Err(Error::DuplicatedSections(last_section_order)),
+							_ => {},
+						};
+
 						last_section_order = section.order();
 					}
 					sections.push(section);
-				}
+				},
 			}
 		}
 
-		let module = Module {
-			magic: u32::from_le_bytes(magic),
-			version: version,
-			sections: sections,
-		};
+		let module = Module { magic: u32::from_le_bytes(magic), version, sections };
 
 		if module.code_section().map(|cs| cs.bodies().len()).unwrap_or(0) !=
 			module.function_section().map(|fs| fs.entries().len()).unwrap_or(0)
 		{
-			return Err(Error::InconsistentCode);
+			return Err(Error::InconsistentCode)
 		}
 
 		Ok(module)
@@ -592,7 +649,7 @@ impl<'a> io::Read for PeekSection<'a> {
 	fn read(&mut self, buf: &mut [u8]) -> io::Result<()> {
 		let available = cmp::min(buf.len(), self.region.len() - self.cursor);
 		if available < buf.len() {
-			return Err(io::Error::UnexpectedEof);
+			return Err(io::Error::UnexpectedEof)
 		}
 
 		let range = self.cursor..self.cursor + buf.len();
@@ -606,7 +663,7 @@ impl<'a> io::Read for PeekSection<'a> {
 /// Returns size of the module in the provided stream.
 pub fn peek_size(source: &[u8]) -> usize {
 	if source.len() < 9 {
-		return 0;
+		return 0
 	}
 
 	let mut cursor = 8;
@@ -615,11 +672,11 @@ pub fn peek_size(source: &[u8]) -> usize {
 			let mut peek_section = PeekSection { cursor: 0, region: &source[cursor..] };
 			let section_id: u8 = match super::VarUint7::deserialize(&mut peek_section) {
 				Ok(res) => res.into(),
-				Err(_) => { break; },
+				Err(_) => break,
 			};
 			let section_len: u32 = match super::VarUint32::deserialize(&mut peek_section) {
 				Ok(res) => res.into(),
-				Err(_) => { break; },
+				Err(_) => break,
 			};
 
 			(peek_section.cursor, section_id, section_len)
@@ -627,15 +684,18 @@ pub fn peek_size(source: &[u8]) -> usize {
 
 		if section_id <= 11 && section_len > 0 {
 			let next_cursor = cursor + new_cursor + section_len as usize;
-			if next_cursor > source.len() {
-				break;
-			} else if next_cursor == source.len() {
-				cursor = next_cursor;
-				break;
+
+			match next_cursor {
+				x if x > source.len() => break,
+				x if x == source.len() => {
+					cursor = next_cursor;
+					break
+				},
+				_ => {},
 			}
 			cursor = next_cursor;
 		} else {
-			break;
+			break
 		}
 	}
 
@@ -644,8 +704,13 @@ pub fn peek_size(source: &[u8]) -> usize {
 
 #[cfg(test)]
 mod integration_tests {
-	use super::super::{deserialize_file, serialize, deserialize_buffer, Section, TypeSection, FunctionSection, ExportSection, CodeSection};
-	use super::Module;
+	use super::{
+		super::{
+			deserialize_buffer, deserialize_file, serialize, CodeSection, ExportSection,
+			FunctionSection, Section, TypeSection,
+		},
+		Module,
+	};
 
 	#[test]
 	fn hello() {
@@ -661,22 +726,23 @@ mod integration_tests {
 		let buf = serialize(module).expect("serialization to succeed");
 
 		let module_new: Module = deserialize_buffer(&buf).expect("deserialization to succeed");
-		let module_old = deserialize_file("./res/cases/v1/test5.wasm").expect("Should be deserialized");
+		let module_old =
+			deserialize_file("./res/cases/v1/test5.wasm").expect("Should be deserialized");
 
 		assert_eq!(module_old.sections().len(), module_new.sections().len());
 	}
 
 	#[test]
 	fn serde_type() {
-		let mut module = deserialize_file("./res/cases/v1/test5.wasm").expect("Should be deserialized");
-		module.sections_mut().retain(|x| {
-			if let &Section::Type(_) = x { true } else { false }
-		});
+		let mut module =
+			deserialize_file("./res/cases/v1/test5.wasm").expect("Should be deserialized");
+		module.sections_mut().retain(|x| matches!(x, &Section::Type(_)));
 
 		let buf = serialize(module).expect("serialization to succeed");
 
 		let module_new: Module = deserialize_buffer(&buf).expect("deserialization to succeed");
-		let module_old = deserialize_file("./res/cases/v1/test5.wasm").expect("Should be deserialized");
+		let module_old =
+			deserialize_file("./res/cases/v1/test5.wasm").expect("Should be deserialized");
 		assert_eq!(
 			module_old.type_section().expect("type section exists").types().len(),
 			module_new.type_section().expect("type section exists").types().len(),
@@ -686,15 +752,15 @@ mod integration_tests {
 
 	#[test]
 	fn serde_import() {
-		let mut module = deserialize_file("./res/cases/v1/test5.wasm").expect("Should be deserialized");
-		module.sections_mut().retain(|x| {
-			if let &Section::Import(_) = x { true } else { false }
-		});
+		let mut module =
+			deserialize_file("./res/cases/v1/test5.wasm").expect("Should be deserialized");
+		module.sections_mut().retain(|x| matches!(x, &Section::Import(_)));
 
 		let buf = serialize(module).expect("serialization to succeed");
 
 		let module_new: Module = deserialize_buffer(&buf).expect("deserialization to succeed");
-		let module_old = deserialize_file("./res/cases/v1/test5.wasm").expect("Should be deserialized");
+		let module_old =
+			deserialize_file("./res/cases/v1/test5.wasm").expect("Should be deserialized");
 		assert_eq!(
 			module_old.import_section().expect("import section exists").entries().len(),
 			module_new.import_section().expect("import section exists").entries().len(),
@@ -704,16 +770,20 @@ mod integration_tests {
 
 	#[test]
 	fn serde_code() {
-		let mut module = deserialize_file("./res/cases/v1/test5.wasm").expect("Should be deserialized");
+		let mut module =
+			deserialize_file("./res/cases/v1/test5.wasm").expect("Should be deserialized");
 		module.sections_mut().retain(|x| {
-			if let &Section::Code(_) = x { return true }
-			if let &Section::Function(_) = x { true } else { false }
+			if let Section::Code(_) = *x {
+				return true
+			}
+			matches!(*x, Section::Function(_))
 		});
 
 		let buf = serialize(module).expect("serialization to succeed");
 
 		let module_new: Module = deserialize_buffer(&buf).expect("deserialization to succeed");
-		let module_old = deserialize_file("./res/cases/v1/test5.wasm").expect("Should be deserialized");
+		let module_old =
+			deserialize_file("./res/cases/v1/test5.wasm").expect("Should be deserialized");
 		assert_eq!(
 			module_old.code_section().expect("code section exists").bodies().len(),
 			module_new.code_section().expect("code section exists").bodies().len(),
@@ -753,7 +823,8 @@ mod integration_tests {
 	fn store() {
 		use super::super::Instruction::*;
 
-		let module = deserialize_file("./res/cases/v1/offset.wasm").expect("Should be deserialized");
+		let module =
+			deserialize_file("./res/cases/v1/offset.wasm").expect("Should be deserialized");
 		let func = &module.code_section().expect("Code section to exist").bodies()[0];
 
 		assert_eq!(func.code().elements().len(), 5);
@@ -772,12 +843,12 @@ mod integration_tests {
 		assert_eq!(peek_size(&buf), buf.len() - 4);
 	}
 
-
 	#[test]
 	fn peek_2() {
 		use super::peek_size;
 
-		let module = deserialize_file("./res/cases/v1/offset.wasm").expect("Should be deserialized");
+		let module =
+			deserialize_file("./res/cases/v1/offset.wasm").expect("Should be deserialized");
 		let mut buf = serialize(module).expect("serialization to succeed");
 
 		buf.extend_from_slice(&[0, 0, 0, 0, 0, 1, 5, 12, 17]);
@@ -789,7 +860,8 @@ mod integration_tests {
 	fn peek_3() {
 		use super::peek_size;
 
-		let module = deserialize_file("./res/cases/v1/peek_sample.wasm").expect("Should be deserialized");
+		let module =
+			deserialize_file("./res/cases/v1/peek_sample.wasm").expect("Should be deserialized");
 		let buf = serialize(module).expect("serialization to succeed");
 
 		assert_eq!(peek_size(&buf), buf.len());
@@ -813,23 +885,47 @@ mod integration_tests {
 
 		let mut found_section = false;
 		for section in module.sections() {
-			match *section {
-				Section::Name(ref name_section) => {
-					let function_name_subsection = name_section
-						.functions()
-						.expect("function_name_subsection should be present");
-					assert_eq!(
-						function_name_subsection.names().get(0).expect("Should be entry #0"),
-						"elog"
-					);
-					assert_eq!(
-						function_name_subsection.names().get(11).expect("Should be entry #0"),
-						"_ZN48_$LT$pwasm_token_contract..Endpoint$LT$T$GT$$GT$3new17hc3ace6dea0978cd9E"
-					);
+			if let Section::Name(ref name_section) = *section {
+				let function_name_subsection =
+					name_section.functions().expect("function_name_subsection should be present");
+				assert_eq!(
+					function_name_subsection.names().get(0).expect("Should be entry #0"),
+					"elog"
+				);
+				assert_eq!(
+					function_name_subsection.names().get(11).expect("Should be entry #0"),
+					"_ZN48_$LT$pwasm_token_contract..Endpoint$LT$T$GT$$GT$3new17hc3ace6dea0978cd9E"
+				);
 
-					found_section = true;
-				},
-				_ => {},
+				found_section = true;
+			}
+		}
+
+		assert!(found_section, "Name section should be present in dedicated example");
+	}
+
+	#[test]
+	fn names_with_global_section() {
+		let module = deserialize_file("./res/cases/v1/global_section.wasm")
+			.expect("Should be deserialized")
+			.parse_names()
+			.expect("Names to be parsed");
+
+		let mut found_section = false;
+		for section in module.sections() {
+			if let Section::Name(ref name_section) = *section {
+				let function_name_subsection =
+					name_section.functions().expect("function_name_subsection should be present");
+				assert_eq!(
+					function_name_subsection.names().get(0).expect("Should be entry #0"),
+					"~lib/builtins/abort"
+				);
+				assert_eq!(
+					function_name_subsection.names().get(11).expect("Should be entry #0"),
+					"~lib/typedarray/Uint8Array#__set"
+				);
+
+				found_section = true;
 			}
 		}
 
@@ -840,132 +936,146 @@ mod integration_tests {
 	#[test]
 	fn shared_memory_flag() {
 		let module = deserialize_file("./res/cases/v1/varuint1_1.wasm");
-		assert_eq!(module.is_ok(), cfg!(feature="atomics"));
+		assert_eq!(module.is_ok(), cfg!(feature = "atomics"));
 	}
 
-
 	#[test]
 	fn memory_space() {
-		let module = deserialize_file("./res/cases/v1/two-mems.wasm").expect("failed to deserialize");
+		let module =
+			deserialize_file("./res/cases/v1/two-mems.wasm").expect("failed to deserialize");
 		assert_eq!(module.memory_space(), 2);
 	}
 
-    #[test]
-    fn add_custom_section() {
-        let mut module = deserialize_file("./res/cases/v1/start_mut.wasm").expect("failed to deserialize");
-        assert!(module.custom_sections().next().is_none());
-        module.set_custom_section("mycustomsection".to_string(), vec![1, 2, 3, 4]);
-        {
-	        let sections = module.custom_sections().collect::<Vec<_>>();
-	        assert_eq!(sections.len(), 1);
-	        assert_eq!(sections[0].name(), "mycustomsection");
-	        assert_eq!(sections[0].payload(), &[1, 2, 3, 4]);
-	    }
+	#[test]
+	fn add_custom_section() {
+		let mut module =
+			deserialize_file("./res/cases/v1/start_mut.wasm").expect("failed to deserialize");
+		assert!(module.custom_sections().next().is_none());
+		module.set_custom_section("mycustomsection".to_string(), vec![1, 2, 3, 4]);
+		{
+			let sections = module.custom_sections().collect::<Vec<_>>();
+			assert_eq!(sections.len(), 1);
+			assert_eq!(sections[0].name(), "mycustomsection");
+			assert_eq!(sections[0].payload(), &[1, 2, 3, 4]);
+		}
 
-        let old_section = module.clear_custom_section("mycustomsection");
-        assert_eq!(old_section.expect("Did not find custom section").payload(), &[1, 2, 3, 4]);
+		let old_section = module.clear_custom_section("mycustomsection");
+		assert_eq!(old_section.expect("Did not find custom section").payload(), &[1, 2, 3, 4]);
 
-        assert!(module.custom_sections().next().is_none());
-    }
+		assert!(module.custom_sections().next().is_none());
+	}
 
-    #[test]
-    fn mut_start() {
-        let mut module = deserialize_file("./res/cases/v1/start_mut.wasm").expect("failed to deserialize");
-        assert_eq!(module.start_section().expect("Did not find any start section"), 1);
-        module.set_start_section(0);
-        assert_eq!(module.start_section().expect("Did not find any start section"), 0);
-        module.clear_start_section();
-        assert_eq!(None, module.start_section());
-    }
+	#[test]
+	fn mut_start() {
+		let mut module =
+			deserialize_file("./res/cases/v1/start_mut.wasm").expect("failed to deserialize");
+		assert_eq!(module.start_section().expect("Did not find any start section"), 1);
+		module.set_start_section(0);
+		assert_eq!(module.start_section().expect("Did not find any start section"), 0);
+		module.clear_start_section();
+		assert_eq!(None, module.start_section());
+	}
 
-    #[test]
-    fn add_start() {
-        let mut module = deserialize_file("./res/cases/v1/start_add.wasm").expect("failed to deserialize");
-        assert!(module.start_section().is_none());
-        module.set_start_section(0);
-        assert_eq!(module.start_section().expect("Did not find any start section"), 0);
+	#[test]
+	fn add_start() {
+		let mut module =
+			deserialize_file("./res/cases/v1/start_add.wasm").expect("failed to deserialize");
+		assert!(module.start_section().is_none());
+		module.set_start_section(0);
+		assert_eq!(module.start_section().expect("Did not find any start section"), 0);
 
-        let sections = module.sections().iter().map(|s| s.order()).collect::<Vec<_>>();
-        assert_eq!(sections, vec![1, 2, 3, 6, 7, 8, 9, 11, 12]);
-    }
+		let sections = module.sections().iter().map(|s| s.order()).collect::<Vec<_>>();
+		assert_eq!(sections, vec![1, 2, 3, 6, 7, 8, 9, 11, 12]);
+	}
 
-    #[test]
-    fn add_start_custom() {
-        let mut module = deserialize_file("./res/cases/v1/start_add_custom.wasm").expect("failed to deserialize");
+	#[test]
+	fn add_start_custom() {
+		let mut module = deserialize_file("./res/cases/v1/start_add_custom.wasm")
+			.expect("failed to deserialize");
 
-        let sections = module.sections().iter().map(|s| s.order()).collect::<Vec<_>>();
-        assert_eq!(sections, vec![1, 2, 3, 6, 7, 9, 11, 12, 0]);
+		let sections = module.sections().iter().map(|s| s.order()).collect::<Vec<_>>();
+		assert_eq!(sections, vec![1, 2, 3, 6, 7, 9, 11, 12, 0]);
 
-        assert!(module.start_section().is_none());
-        module.set_start_section(0);
-        assert_eq!(module.start_section().expect("Dorder not find any start section"), 0);
+		assert!(module.start_section().is_none());
+		module.set_start_section(0);
+		assert_eq!(module.start_section().expect("Dorder not find any start section"), 0);
 
-        let sections = module.sections().iter().map(|s| s.order()).collect::<Vec<_>>();
-        assert_eq!(sections, vec![1, 2, 3, 6, 7, 8, 9, 11, 12, 0]);
-    }
+		let sections = module.sections().iter().map(|s| s.order()).collect::<Vec<_>>();
+		assert_eq!(sections, vec![1, 2, 3, 6, 7, 8, 9, 11, 12, 0]);
+	}
 
-    #[test]
-    fn names_section_present() {
-        let mut module = deserialize_file("./res/cases/v1/names.wasm").expect("failed to deserialize");
+	#[test]
+	fn names_section_present() {
+		let mut module =
+			deserialize_file("./res/cases/v1/names.wasm").expect("failed to deserialize");
 
-        // Before parsing
-        assert!(module.names_section().is_none());
-        assert!(module.names_section_mut().is_none());
-        assert!(module.has_names_section());
+		// Before parsing
+		assert!(module.names_section().is_none());
+		assert!(module.names_section_mut().is_none());
+		assert!(module.has_names_section());
 
-        // After parsing
-        let mut module = module.parse_names().expect("failed to parse names section");
-        assert!(module.names_section().is_some());
-        assert!(module.names_section_mut().is_some());
-        assert!(module.has_names_section());
-    }
+		// After parsing
+		let mut module = module.parse_names().expect("failed to parse names section");
+		assert!(module.names_section().is_some());
+		assert!(module.names_section_mut().is_some());
+		assert!(module.has_names_section());
+	}
 
-    #[test]
-    fn names_section_not_present() {
-        let mut module = deserialize_file("./res/cases/v1/test.wasm").expect("failed to deserialize");
+	#[test]
+	fn names_section_not_present() {
+		let mut module =
+			deserialize_file("./res/cases/v1/test.wasm").expect("failed to deserialize");
 
-        // Before parsing
-        assert!(module.names_section().is_none());
-        assert!(module.names_section_mut().is_none());
-        assert!(!module.has_names_section());
+		// Before parsing
+		assert!(module.names_section().is_none());
+		assert!(module.names_section_mut().is_none());
+		assert!(!module.has_names_section());
 
-        // After parsing
-        let mut module = module.parse_names().expect("failed to parse names section");
-        assert!(module.names_section().is_none());
-        assert!(module.names_section_mut().is_none());
-        assert!(!module.has_names_section());
-    }
+		// After parsing
+		let mut module = module.parse_names().expect("failed to parse names section");
+		assert!(module.names_section().is_none());
+		assert!(module.names_section_mut().is_none());
+		assert!(!module.has_names_section());
+	}
 
-    #[test]
-    fn insert_sections() {
-        let mut module = Module::default();
+	#[test]
+	fn insert_sections() {
+		let mut module = Module::default();
 
-        assert!(module.insert_section(Section::Function(FunctionSection::with_entries(vec![]))).is_ok());
-        // Duplicate.
-        assert!(module.insert_section(Section::Function(FunctionSection::with_entries(vec![]))).is_err());
+		assert!(module
+			.insert_section(Section::Function(FunctionSection::with_entries(vec![])))
+			.is_ok());
+		// Duplicate.
+		assert!(module
+			.insert_section(Section::Function(FunctionSection::with_entries(vec![])))
+			.is_err());
 
-        assert!(module.insert_section(Section::Type(TypeSection::with_types(vec![]))).is_ok());
-        // Duplicate.
-        assert!(module.insert_section(Section::Type(TypeSection::with_types(vec![]))).is_err());
+		assert!(module.insert_section(Section::Type(TypeSection::with_types(vec![]))).is_ok());
+		// Duplicate.
+		assert!(module.insert_section(Section::Type(TypeSection::with_types(vec![]))).is_err());
 
-        assert!(module.insert_section(Section::Export(ExportSection::with_entries(vec![]))).is_ok());
-        // Duplicate.
-        assert!(module.insert_section(Section::Export(ExportSection::with_entries(vec![]))).is_err());
+		assert!(module
+			.insert_section(Section::Export(ExportSection::with_entries(vec![])))
+			.is_ok());
+		// Duplicate.
+		assert!(module
+			.insert_section(Section::Export(ExportSection::with_entries(vec![])))
+			.is_err());
 
-        assert!(module.insert_section(Section::Code(CodeSection::with_bodies(vec![]))).is_ok());
-        // Duplicate.
-        assert!(module.insert_section(Section::Code(CodeSection::with_bodies(vec![]))).is_err());
+		assert!(module.insert_section(Section::Code(CodeSection::with_bodies(vec![]))).is_ok());
+		// Duplicate.
+		assert!(module.insert_section(Section::Code(CodeSection::with_bodies(vec![]))).is_err());
 
-        // Try serialisation roundtrip to check well-orderedness.
-        let serialized = serialize(module).expect("serialization to succeed");
-        assert!(deserialize_buffer::<Module>(&serialized).is_ok());
-    }
+		// Try serialisation roundtrip to check well-orderedness.
+		let serialized = serialize(module).expect("serialization to succeed");
+		assert!(deserialize_buffer::<Module>(&serialized).is_ok());
+	}
 
-    #[test]
-    fn serialization_roundtrip() {
-        let module = deserialize_file("./res/cases/v1/test.wasm").expect("failed to deserialize");
-        let module_copy = module.clone().to_bytes().expect("failed to serialize");
-        let module_copy = Module::from_bytes(&module_copy).expect("failed to deserialize");
-        assert_eq!(module, module_copy);
-    }
+	#[test]
+	fn serialization_roundtrip() {
+		let module = deserialize_file("./res/cases/v1/test.wasm").expect("failed to deserialize");
+		let module_copy = module.clone().into_bytes().expect("failed to serialize");
+		let module_copy = Module::from_bytes(&module_copy).expect("failed to deserialize");
+		assert_eq!(module, module_copy);
+	}
 }
