@@ -1,9 +1,9 @@
-use crate::rust::{fmt, vec::Vec};
-use crate::io;
 use super::{
-	Deserialize, Serialize, Error, VarUint7, VarInt7, VarUint1, CountedList,
-	CountedListWriter, VarUint32,
+	CountedList, CountedListWriter, Deserialize, Error, Serialize, VarInt32, VarInt7, VarUint7,
 };
+use crate::io;
+use alloc::vec::Vec;
+use core::fmt;
 
 /// Type definition in types section. Currently can be only of the function type.
 #[derive(Debug, Clone, PartialEq, Hash, Eq)]
@@ -25,7 +25,7 @@ impl Serialize for Type {
 
 	fn serialize<W: io::Write>(self, writer: &mut W) -> Result<(), Self::Error> {
 		match self {
-			Type::Function(fn_type) => fn_type.serialize(writer)
+			Type::Function(fn_type) => fn_type.serialize(writer),
 		}
 	}
 }
@@ -41,6 +41,7 @@ pub enum ValueType {
 	F32,
 	/// 64-bit float
 	F64,
+	#[cfg(feature = "simd")]
 	/// 128-bit SIMD register
 	V128,
 }
@@ -56,6 +57,7 @@ impl Deserialize for ValueType {
 			-0x02 => Ok(ValueType::I64),
 			-0x03 => Ok(ValueType::F32),
 			-0x04 => Ok(ValueType::F64),
+			#[cfg(feature = "simd")]
 			-0x05 => Ok(ValueType::V128),
 			_ => Err(Error::UnknownValueType(val.into())),
 		}
@@ -71,8 +73,10 @@ impl Serialize for ValueType {
 			ValueType::I64 => -0x02,
 			ValueType::F32 => -0x03,
 			ValueType::F64 => -0x04,
+			#[cfg(feature = "simd")]
 			ValueType::V128 => -0x05,
-		}.into();
+		}
+		.into();
 		val.serialize(writer)?;
 		Ok(())
 	}
@@ -85,6 +89,7 @@ impl fmt::Display for ValueType {
 			ValueType::I64 => write!(f, "i64"),
 			ValueType::F32 => write!(f, "f32"),
 			ValueType::F64 => write!(f, "f64"),
+			#[cfg(feature = "simd")]
 			ValueType::V128 => write!(f, "v128"),
 		}
 	}
@@ -93,26 +98,36 @@ impl fmt::Display for ValueType {
 /// Block type which is basically `ValueType` + NoResult (to define blocks that have no return type)
 #[derive(Clone, Copy, Debug, PartialEq, Eq, Hash)]
 pub enum BlockType {
-	/// Value-type specified block type
-	Value(ValueType),
 	/// No specified block type
 	NoResult,
+	/// Inline value type.
+	Value(ValueType),
+	/// Reference to a signature.
+	#[cfg(feature = "multi_value")]
+	TypeIndex(u32),
 }
 
 impl Deserialize for BlockType {
 	type Error = Error;
 
 	fn deserialize<R: io::Read>(reader: &mut R) -> Result<Self, Self::Error> {
-		let val = VarInt7::deserialize(reader)?;
+		let val = VarInt32::deserialize(reader)?;
 
 		match val.into() {
+			-0x40 => Ok(BlockType::NoResult),
 			-0x01 => Ok(BlockType::Value(ValueType::I32)),
 			-0x02 => Ok(BlockType::Value(ValueType::I64)),
 			-0x03 => Ok(BlockType::Value(ValueType::F32)),
 			-0x04 => Ok(BlockType::Value(ValueType::F64)),
-			0x7b => Ok(BlockType::Value(ValueType::V128)),
-			-0x40 => Ok(BlockType::NoResult),
-			_ => Err(Error::UnknownValueType(val.into())),
+			#[cfg(feature = "simd")]
+			-0x05 => Ok(BlockType::Value(ValueType::V128)),
+			#[cfg(feature = "multi_value")]
+			idx => {
+				let idx = idx.try_into().map_err(|_| Error::UnknownBlockType(idx))?;
+				Ok(BlockType::TypeIndex(idx))
+			},
+			#[cfg(not(feature = "multi_value"))]
+			_ => Err(Error::UnknownBlockType(val.into())),
 		}
 	}
 }
@@ -121,14 +136,18 @@ impl Serialize for BlockType {
 	type Error = Error;
 
 	fn serialize<W: io::Write>(self, writer: &mut W) -> Result<(), Self::Error> {
-		let val: VarInt7 = match self {
-			BlockType::NoResult => -0x40i8,
+		let val: VarInt32 = match self {
+			BlockType::NoResult => -0x40,
 			BlockType::Value(ValueType::I32) => -0x01,
 			BlockType::Value(ValueType::I64) => -0x02,
 			BlockType::Value(ValueType::F32) => -0x03,
 			BlockType::Value(ValueType::F64) => -0x04,
-			BlockType::Value(ValueType::V128) => 0x7b,
-		}.into();
+			#[cfg(feature = "simd")]
+			BlockType::Value(ValueType::V128) => -0x05,
+			#[cfg(feature = "multi_value")]
+			BlockType::TypeIndex(idx) => idx as i32,
+		}
+		.into();
 		val.serialize(writer)?;
 		Ok(())
 	}
@@ -139,38 +158,40 @@ impl Serialize for BlockType {
 pub struct FunctionType {
 	form: u8,
 	params: Vec<ValueType>,
-	return_type: Option<ValueType>,
+	results: Vec<ValueType>,
 }
 
 impl Default for FunctionType {
 	fn default() -> Self {
-		FunctionType {
-			form: 0x60,
-			params: Vec::new(),
-			return_type: None,
-		}
+		FunctionType { form: 0x60, params: Vec::new(), results: Vec::new() }
 	}
 }
 
 impl FunctionType {
-	/// New function type given the signature in-params(`params`) and return type (`return_type`)
-	pub fn new(params: Vec<ValueType>, return_type: Option<ValueType>) -> Self {
-		FunctionType {
-			params: params,
-			return_type: return_type,
-			..Default::default()
-		}
+	/// New function type given the params and results as vectors
+	pub fn new(params: Vec<ValueType>, results: Vec<ValueType>) -> Self {
+		FunctionType { form: 0x60, params, results }
 	}
 	/// Function form (currently only valid value is `0x60`)
-	pub fn form(&self) -> u8 { self.form }
+	pub fn form(&self) -> u8 {
+		self.form
+	}
 	/// Parameters in the function signature.
-	pub fn params(&self) -> &[ValueType] { &self.params }
+	pub fn params(&self) -> &[ValueType] {
+		&self.params
+	}
 	/// Mutable parameters in the function signature.
-	pub fn params_mut(&mut self) -> &mut Vec<ValueType> { &mut self.params }
-	/// Return type in the function signature, if any.
-	pub fn return_type(&self) -> Option<ValueType> { self.return_type }
+	pub fn params_mut(&mut self) -> &mut Vec<ValueType> {
+		&mut self.params
+	}
+	/// Results in the function signature, if any.
+	pub fn results(&self) -> &[ValueType] {
+		&self.results
+	}
 	/// Mutable type in the function signature, if any.
-	pub fn return_type_mut(&mut self) -> &mut Option<ValueType> { &mut self.return_type }
+	pub fn results_mut(&mut self) -> &mut Vec<ValueType> {
+		&mut self.results
+	}
 }
 
 impl Deserialize for FunctionType {
@@ -180,26 +201,20 @@ impl Deserialize for FunctionType {
 		let form: u8 = VarUint7::deserialize(reader)?.into();
 
 		if form != 0x60 {
-			return Err(Error::UnknownFunctionForm(form));
+			return Err(Error::UnknownFunctionForm(form))
 		}
 
 		let params: Vec<ValueType> = CountedList::deserialize(reader)?.into_inner();
+		let results: Vec<ValueType> = CountedList::deserialize(reader)?.into_inner();
 
-		let return_types: u32 = VarUint32::deserialize(reader)?.into();
+		#[cfg(not(feature = "multi_value"))]
+		if results.len() > 1 {
+			return Err(Error::Other(
+				"Enable the multi_value feature to deserialize more than one function result",
+			))
+		}
 
-		let return_type = if return_types == 1 {
-			Some(ValueType::deserialize(reader)?)
-		} else if return_types == 0 {
-			None
-		} else {
-			return Err(Error::Other("Return types length should be 0 or 1"));
-		};
-
-		Ok(FunctionType {
-			form: form,
-			params: params,
-			return_type: return_type,
-		})
+		Ok(FunctionType { form, params, results })
 	}
 }
 
@@ -209,19 +224,17 @@ impl Serialize for FunctionType {
 	fn serialize<W: io::Write>(self, writer: &mut W) -> Result<(), Self::Error> {
 		VarUint7::from(self.form).serialize(writer)?;
 
-		let data = self.params;
-		let counted_list = CountedListWriter::<ValueType, _>(
-			data.len(),
-			data.into_iter().map(Into::into),
+		let params_counted_list = CountedListWriter::<ValueType, _>(
+			self.params.len(),
+			self.params.into_iter().map(Into::into),
 		);
-		counted_list.serialize(writer)?;
+		params_counted_list.serialize(writer)?;
 
-		if let Some(return_type) = self.return_type {
-			VarUint1::from(true).serialize(writer)?;
-			return_type.serialize(writer)?;
-		} else {
-			VarUint1::from(false).serialize(writer)?;
-		}
+		let results_counted_list = CountedListWriter::<ValueType, _>(
+			self.results.len(),
+			self.results.into_iter().map(Into::into),
+		);
+		results_counted_list.serialize(writer)?;
 
 		Ok(())
 	}
@@ -253,7 +266,8 @@ impl Serialize for TableElementType {
 	fn serialize<W: io::Write>(self, writer: &mut W) -> Result<(), Self::Error> {
 		let val: VarInt7 = match self {
 			TableElementType::AnyFunc => -0x10,
-		}.into();
+		}
+		.into();
 		val.serialize(writer)?;
 		Ok(())
 	}
